#! /usr/bin/env python

#David Shean
#dshean@gmail.com

#Load custom color ramp from cpt
#gmtColormap function from: http://wiki.scipy.org/Cookbook/Matplotlib/Loading_a_colormap_dynamically

#To use:
#import gmtColormap
#cpt_rainbow = gmtColormap.get_rainbow()

def get_rainbow(rev=False):
    import os
    import matplotlib.colors 
    #Check for bundled cpt file in script directory
    fn_path = os.path.dirname(os.path.realpath(__file__))
    fn = os.path.join(fn_path,'rainbow.cpt')
    if not os.path.isfile(fn):
        #Create a temporary version
        fn = write_rainbow_cpt()
    d = gmtColormap(fn, reverse=rev)
    name = 'cpt_rainbow'
    if rev:
        name = 'cpt_rainbow_r'
    cmap = matplotlib.colors.LinearSegmentedColormap(name, d)
    return cmap

def gmtColormap(fileName,GMTPath = None, reverse=False):
      import colorsys
      import numpy as np 
      #if type(GMTPath) == type(None):
      #    filePath = "/usr/local/cmaps/"+ fileName+".cpt"
      #else:
      #    filePath = GMTPath+"/"+ fileName +".cpt"
      filePath = fileName
      try:
          f = open(filePath)
      except:
          print "file ",filePath, "not found"
          return None

      lines = f.readlines()
      f.close()
      
      x = []
      r = []
      g = []
      b = []
      colorModel = "RGB"
      for l in lines:
          ls = l.split()
          if l[0] == "#":
             if ls[-1] == "HSV":
                 colorModel = "HSV"
                 continue
             else:
                 continue
          if ls[0] == "B" or ls[0] == "F" or ls[0] == "N":
             pass
          else:
              x.append(float(ls[0]))
              r.append(float(ls[1]))
              g.append(float(ls[2]))
              b.append(float(ls[3]))
              xtemp = float(ls[4])
              rtemp = float(ls[5])
              gtemp = float(ls[6])
              btemp = float(ls[7])

      x.append(xtemp)
      r.append(rtemp)
      g.append(gtemp)
      b.append(btemp)

      nTable = len(r)
      x = np.array( x , np.float)
      r = np.array( r , np.float)
      g = np.array( g , np.float)
      b = np.array( b , np.float)

      if reverse:
        r = r[::-1]
        g = g[::-1]
        b = b[::-1]

      if colorModel == "HSV":
         for i in range(r.shape[0]):
             rr,gg,bb = colorsys.hsv_to_rgb(r[i]/360.,g[i],b[i])
             r[i] = rr ; g[i] = gg ; b[i] = bb
      if colorModel == "HSV":
         for i in range(r.shape[0]):
             rr,gg,bb = colorsys.hsv_to_rgb(r[i]/360.,g[i],b[i])
             r[i] = rr ; g[i] = gg ; b[i] = bb
      if colorModel == "RGB":
          r = r/255.
          g = g/255.
          b = b/255.
      xNorm = (x - x[0])/(x[-1] - x[0])

      red = []
      blue = []
      green = []
      for i in range(len(x)):
          red.append([xNorm[i],r[i],r[i]])
          green.append([xNorm[i],g[i],g[i]])
          blue.append([xNorm[i],b[i],b[i]])
      colorDict = {"red":red, "green":green, "blue":blue}
      return (colorDict)

def write_rainbow_cpt():
    import tempfile
    fn = None
    with tempfile.NamedTemporaryFile(delete=False) as temp:
        temp.write("""\
# ../cpt/ocw/rainbow.cpt
# autogenerated GMT palette "<stdin>"
# cptutils version 1.31, Mon Jan 26 23:28:32 2009
# COLOR_MODEL = RGB
1.000000e+00 144   0 111 2.000000e+00 141   0 114
2.000000e+00 141   0 114 3.000000e+00 138   0 117
3.000000e+00 138   0 117 4.000000e+00 135   0 120
4.000000e+00 135   0 120 5.000000e+00 132   0 123
5.000000e+00 132   0 123 6.000000e+00 129   0 126
6.000000e+00 129   0 126 7.000000e+00 126   0 129
7.000000e+00 126   0 129 8.000000e+00 123   0 132
8.000000e+00 123   0 132 9.000000e+00 120   0 135
9.000000e+00 120   0 135 1.000000e+01 117   0 138
1.000000e+01 117   0 138 1.100000e+01 114   0 141
1.100000e+01 114   0 141 1.200000e+01 111   0 144
1.200000e+01 111   0 144 1.300000e+01 108   0 147
1.300000e+01 108   0 147 1.400000e+01 105   0 150
1.400000e+01 105   0 150 1.500000e+01 102   0 153
1.500000e+01 102   0 153 1.600000e+01  99   0 156
1.600000e+01  99   0 156 1.700000e+01  96   0 159
1.700000e+01  96   0 159 1.800000e+01  93   0 162
1.800000e+01  93   0 162 1.900000e+01  90   0 165
1.900000e+01  90   0 165 2.000000e+01  87   0 168
2.000000e+01  87   0 168 2.100000e+01  84   0 171
2.100000e+01  84   0 171 2.200000e+01  81   0 174
2.200000e+01  81   0 174 2.300000e+01  78   0 177
2.300000e+01  78   0 177 2.400000e+01  75   0 180
2.400000e+01  75   0 180 2.500000e+01  72   0 183
2.500000e+01  72   0 183 2.600000e+01  69   0 186
2.600000e+01  69   0 186 2.700000e+01  66   0 189
2.700000e+01  66   0 189 2.800000e+01  63   0 192
2.800000e+01  63   0 192 2.900000e+01  60   0 195
2.900000e+01  60   0 195 3.000000e+01  57   0 198
3.000000e+01  57   0 198 3.100000e+01  54   0 201
3.100000e+01  54   0 201 3.200000e+01  51   0 204
3.200000e+01  51   0 204 3.300000e+01  48   0 207
3.300000e+01  48   0 207 3.400000e+01  45   0 210
3.400000e+01  45   0 210 3.500000e+01  42   0 213
3.500000e+01  42   0 213 3.600000e+01  39   0 216
3.600000e+01  39   0 216 3.700000e+01  36   0 219
3.700000e+01  36   0 219 3.800000e+01  33   0 222
3.800000e+01  33   0 222 3.900000e+01  30   0 225
3.900000e+01  30   0 225 4.000000e+01  27   0 228
4.000000e+01  27   0 228 4.100000e+01  24   0 231
4.100000e+01  24   0 231 4.200000e+01  21   0 234
4.200000e+01  21   0 234 4.300000e+01  18   0 237
4.300000e+01  18   0 237 4.400000e+01  15   0 240
4.400000e+01  15   0 240 4.500000e+01  12   0 243
4.500000e+01  12   0 243 4.600000e+01   9   0 246
4.600000e+01   9   0 246 4.700000e+01   6   0 249
4.700000e+01   6   0 249 4.800000e+01   0   0 252
4.800000e+01   0   0 252 4.900000e+01   0   0 255
4.900000e+01   0   0 255 5.000000e+01   0   5 255
5.000000e+01   0   5 255 5.100000e+01   0  10 255
5.100000e+01   0  10 255 5.200000e+01   0  16 255
5.200000e+01   0  16 255 5.300000e+01   0  21 255
5.300000e+01   0  21 255 5.400000e+01   0  26 255
5.400000e+01   0  26 255 5.500000e+01   0  32 255
5.500000e+01   0  32 255 5.600000e+01   0  37 255
5.600000e+01   0  37 255 5.700000e+01   0  42 255
5.700000e+01   0  42 255 5.800000e+01   0  48 255
5.800000e+01   0  48 255 5.900000e+01   0  53 255
5.900000e+01   0  53 255 6.000000e+01   0  58 255
6.000000e+01   0  58 255 6.100000e+01   0  64 255
6.100000e+01   0  64 255 6.200000e+01   0  69 255
6.200000e+01   0  69 255 6.300000e+01   0  74 255
6.300000e+01   0  74 255 6.400000e+01   0  80 255
6.400000e+01   0  80 255 6.500000e+01   0  85 255
6.500000e+01   0  85 255 6.600000e+01   0  90 255
6.600000e+01   0  90 255 6.700000e+01   0  96 255
6.700000e+01   0  96 255 6.800000e+01   0 101 255
6.800000e+01   0 101 255 6.900000e+01   0 106 255
6.900000e+01   0 106 255 7.000000e+01   0 112 255
7.000000e+01   0 112 255 7.100000e+01   0 117 255
7.100000e+01   0 117 255 7.200000e+01   0 122 255
7.200000e+01   0 122 255 7.300000e+01   0 128 255
7.300000e+01   0 128 255 7.400000e+01   0 133 255
7.400000e+01   0 133 255 7.500000e+01   0 138 255
7.500000e+01   0 138 255 7.600000e+01   0 144 255
7.600000e+01   0 144 255 7.700000e+01   0 149 255
7.700000e+01   0 149 255 7.800000e+01   0 154 255
7.800000e+01   0 154 255 7.900000e+01   0 160 255
7.900000e+01   0 160 255 8.000000e+01   0 165 255
8.000000e+01   0 165 255 8.100000e+01   0 170 255
8.100000e+01   0 170 255 8.200000e+01   0 176 255
8.200000e+01   0 176 255 8.300000e+01   0 181 255
8.300000e+01   0 181 255 8.400000e+01   0 186 255
8.400000e+01   0 186 255 8.500000e+01   0 192 255
8.500000e+01   0 192 255 8.600000e+01   0 197 255
8.600000e+01   0 197 255 8.700000e+01   0 202 255
8.700000e+01   0 202 255 8.800000e+01   0 208 255
8.800000e+01   0 208 255 8.900000e+01   0 213 255
8.900000e+01   0 213 255 9.000000e+01   0 218 255
9.000000e+01   0 218 255 9.100000e+01   0 224 255
9.100000e+01   0 224 255 9.200000e+01   0 229 255
9.200000e+01   0 229 255 9.300000e+01   0 234 255
9.300000e+01   0 234 255 9.400000e+01   0 240 255
9.400000e+01   0 240 255 9.500000e+01   0 245 255
9.500000e+01   0 245 255 9.600000e+01   0 250 255
9.600000e+01   0 250 255 9.700000e+01   0 255 255
9.700000e+01   0 255 255 9.800000e+01   0 255 247
9.800000e+01   0 255 247 9.900000e+01   0 255 239
9.900000e+01   0 255 239 1.000000e+02   0 255 231
1.000000e+02   0 255 231 1.010000e+02   0 255 223
1.010000e+02   0 255 223 1.020000e+02   0 255 215
1.020000e+02   0 255 215 1.030000e+02   0 255 207
1.030000e+02   0 255 207 1.040000e+02   0 255 199
1.040000e+02   0 255 199 1.050000e+02   0 255 191
1.050000e+02   0 255 191 1.060000e+02   0 255 183
1.060000e+02   0 255 183 1.070000e+02   0 255 175
1.070000e+02   0 255 175 1.080000e+02   0 255 167
1.080000e+02   0 255 167 1.090000e+02   0 255 159
1.090000e+02   0 255 159 1.100000e+02   0 255 151
1.100000e+02   0 255 151 1.110000e+02   0 255 143
1.110000e+02   0 255 143 1.120000e+02   0 255 135
1.120000e+02   0 255 135 1.130000e+02   0 255 127
1.130000e+02   0 255 127 1.140000e+02   0 255 119
1.140000e+02   0 255 119 1.150000e+02   0 255 111
1.150000e+02   0 255 111 1.160000e+02   0 255 103
1.160000e+02   0 255 103 1.170000e+02   0 255  95
1.170000e+02   0 255  95 1.180000e+02   0 255  87
1.180000e+02   0 255  87 1.190000e+02   0 255  79
1.190000e+02   0 255  79 1.200000e+02   0 255  71
1.200000e+02   0 255  71 1.210000e+02   0 255  63
1.210000e+02   0 255  63 1.220000e+02   0 255  55
1.220000e+02   0 255  55 1.230000e+02   0 255  47
1.230000e+02   0 255  47 1.240000e+02   0 255  39
1.240000e+02   0 255  39 1.250000e+02   0 255  31
1.250000e+02   0 255  31 1.260000e+02   0 255  23
1.260000e+02   0 255  23 1.270000e+02   0 255  15
1.270000e+02   0 255  15 1.280000e+02   0 255   0
1.280000e+02   0 255   0 1.290000e+02   8 255   0
1.290000e+02   8 255   0 1.300000e+02  16 255   0
1.300000e+02  16 255   0 1.310000e+02  24 255   0
1.310000e+02  24 255   0 1.320000e+02  32 255   0
1.320000e+02  32 255   0 1.330000e+02  40 255   0
1.330000e+02  40 255   0 1.340000e+02  48 255   0
1.340000e+02  48 255   0 1.350000e+02  56 255   0
1.350000e+02  56 255   0 1.360000e+02  64 255   0
1.360000e+02  64 255   0 1.370000e+02  72 255   0
1.370000e+02  72 255   0 1.380000e+02  80 255   0
1.380000e+02  80 255   0 1.390000e+02  88 255   0
1.390000e+02  88 255   0 1.400000e+02  96 255   0
1.400000e+02  96 255   0 1.410000e+02 104 255   0
1.410000e+02 104 255   0 1.420000e+02 112 255   0
1.420000e+02 112 255   0 1.430000e+02 120 255   0
1.430000e+02 120 255   0 1.440000e+02 128 255   0
1.440000e+02 128 255   0 1.450000e+02 136 255   0
1.450000e+02 136 255   0 1.460000e+02 144 255   0
1.460000e+02 144 255   0 1.470000e+02 152 255   0
1.470000e+02 152 255   0 1.480000e+02 160 255   0
1.480000e+02 160 255   0 1.490000e+02 168 255   0
1.490000e+02 168 255   0 1.500000e+02 176 255   0
1.500000e+02 176 255   0 1.510000e+02 184 255   0
1.510000e+02 184 255   0 1.520000e+02 192 255   0
1.520000e+02 192 255   0 1.530000e+02 200 255   0
1.530000e+02 200 255   0 1.540000e+02 208 255   0
1.540000e+02 208 255   0 1.550000e+02 216 255   0
1.550000e+02 216 255   0 1.560000e+02 224 255   0
1.560000e+02 224 255   0 1.570000e+02 232 255   0
1.570000e+02 232 255   0 1.580000e+02 240 255   0
1.580000e+02 240 255   0 1.590000e+02 248 255   0
1.590000e+02 248 255   0 1.600000e+02 255 255   0
1.600000e+02 255 255   0 1.610000e+02 255 251   0
1.610000e+02 255 251   0 1.620000e+02 255 247   0
1.620000e+02 255 247   0 1.630000e+02 255 243   0
1.630000e+02 255 243   0 1.640000e+02 255 239   0
1.640000e+02 255 239   0 1.650000e+02 255 235   0
1.650000e+02 255 235   0 1.660000e+02 255 231   0
1.660000e+02 255 231   0 1.670000e+02 255 227   0
1.670000e+02 255 227   0 1.680000e+02 255 223   0
1.680000e+02 255 223   0 1.690000e+02 255 219   0
1.690000e+02 255 219   0 1.700000e+02 255 215   0
1.700000e+02 255 215   0 1.710000e+02 255 211   0
1.710000e+02 255 211   0 1.720000e+02 255 207   0
1.720000e+02 255 207   0 1.730000e+02 255 203   0
1.730000e+02 255 203   0 1.740000e+02 255 199   0
1.740000e+02 255 199   0 1.750000e+02 255 195   0
1.750000e+02 255 195   0 1.760000e+02 255 191   0
1.760000e+02 255 191   0 1.770000e+02 255 187   0
1.770000e+02 255 187   0 1.780000e+02 255 183   0
1.780000e+02 255 183   0 1.790000e+02 255 179   0
1.790000e+02 255 179   0 1.800000e+02 255 175   0
1.800000e+02 255 175   0 1.810000e+02 255 171   0
1.810000e+02 255 171   0 1.820000e+02 255 167   0
1.820000e+02 255 167   0 1.830000e+02 255 163   0
1.830000e+02 255 163   0 1.840000e+02 255 159   0
1.840000e+02 255 159   0 1.850000e+02 255 155   0
1.850000e+02 255 155   0 1.860000e+02 255 151   0
1.860000e+02 255 151   0 1.870000e+02 255 147   0
1.870000e+02 255 147   0 1.880000e+02 255 143   0
1.880000e+02 255 143   0 1.890000e+02 255 139   0
1.890000e+02 255 139   0 1.900000e+02 255 135   0
1.900000e+02 255 135   0 1.910000e+02 255 131   0
1.910000e+02 255 131   0 1.920000e+02 255 127   0
1.920000e+02 255 127   0 1.930000e+02 255 123   0
1.930000e+02 255 123   0 1.940000e+02 255 119   0
1.940000e+02 255 119   0 1.950000e+02 255 115   0
1.950000e+02 255 115   0 1.960000e+02 255 111   0
1.960000e+02 255 111   0 1.970000e+02 255 107   0
1.970000e+02 255 107   0 1.980000e+02 255 103   0
1.980000e+02 255 103   0 1.990000e+02 255  99   0
1.990000e+02 255  99   0 2.000000e+02 255  95   0
2.000000e+02 255  95   0 2.010000e+02 255  91   0
2.010000e+02 255  91   0 2.020000e+02 255  87   0
2.020000e+02 255  87   0 2.030000e+02 255  83   0
2.030000e+02 255  83   0 2.040000e+02 255  79   0
2.040000e+02 255  79   0 2.050000e+02 255  75   0
2.050000e+02 255  75   0 2.060000e+02 255  71   0
2.060000e+02 255  71   0 2.070000e+02 255  67   0
2.070000e+02 255  67   0 2.080000e+02 255  63   0
2.080000e+02 255  63   0 2.090000e+02 255  59   0
2.090000e+02 255  59   0 2.100000e+02 255  55   0
2.100000e+02 255  55   0 2.110000e+02 255  51   0
2.110000e+02 255  51   0 2.120000e+02 255  47   0
2.120000e+02 255  47   0 2.130000e+02 255  43   0
2.130000e+02 255  43   0 2.140000e+02 255  39   0
2.140000e+02 255  39   0 2.150000e+02 255  35   0
2.150000e+02 255  35   0 2.160000e+02 255  31   0
2.160000e+02 255  31   0 2.170000e+02 255  27   0
2.170000e+02 255  27   0 2.180000e+02 255  23   0
2.180000e+02 255  23   0 2.190000e+02 255  19   0
2.190000e+02 255  19   0 2.200000e+02 255  15   0
2.200000e+02 255  15   0 2.210000e+02 255  11   0
2.210000e+02 255  11   0 2.220000e+02 255   7   0
2.220000e+02 255   7   0 2.230000e+02 255   3   0
2.230000e+02 255   3   0 2.240000e+02 255   0   0
2.240000e+02 255   0   0 2.250000e+02 250   0   0
2.250000e+02 250   0   0 2.260000e+02 245   0   0
2.260000e+02 245   0   0 2.270000e+02 240   0   0
2.270000e+02 240   0   0 2.280000e+02 235   0   0
2.280000e+02 235   0   0 2.290000e+02 230   0   0
2.290000e+02 230   0   0 2.300000e+02 225   0   0
2.300000e+02 225   0   0 2.310000e+02 220   0   0
2.310000e+02 220   0   0 2.320000e+02 215   0   0
2.320000e+02 215   0   0 2.330000e+02 210   0   0
2.330000e+02 210   0   0 2.340000e+02 205   0   0
2.340000e+02 205   0   0 2.350000e+02 200   0   0
2.350000e+02 200   0   0 2.360000e+02 195   0   0
2.360000e+02 195   0   0 2.370000e+02 190   0   0
2.370000e+02 190   0   0 2.380000e+02 185   0   0
2.380000e+02 185   0   0 2.390000e+02 180   0   0
2.390000e+02 180   0   0 2.400000e+02 175   0   0
2.400000e+02 175   0   0 2.410000e+02 170   0   0
2.410000e+02 170   0   0 2.420000e+02 165   0   0
2.420000e+02 165   0   0 2.430000e+02 160   0   0
2.430000e+02 160   0   0 2.440000e+02 155   0   0
2.440000e+02 155   0   0 2.450000e+02 150   0   0
2.450000e+02 150   0   0 2.460000e+02 145   0   0
2.460000e+02 145   0   0 2.470000e+02 140   0   0
2.470000e+02 140   0   0 2.480000e+02 135   0   0
2.480000e+02 135   0   0 2.490000e+02 130   0   0
2.490000e+02 130   0   0 2.500000e+02 125   0   0
2.500000e+02 125   0   0 2.510000e+02 120   0   0
2.510000e+02 120   0   0 2.520000e+02 115   0   0
2.520000e+02 115   0   0 2.530000e+02 110   0   0
2.530000e+02 110   0   0 2.540000e+02 105   0   0
B   0   0   0
F 255 255 255
N 255   0   0""")
        temp.flush()
        temp.close()
        fn = temp.name
    return fn 
